package GUI;

import DB.Competition;
import DB.Slot;
import Exceptions.NotFoundInTable;
import GUI.Agenda.Agenda;
import GUI.Theme.PrimaryContainedButtonUI;
import GUI.Types.ProjectionType;
import com.intellij.uiDesigner.core.GridConstraints;
import com.intellij.uiDesigner.core.GridLayoutManager;
import com.intellij.uiDesigner.core.Spacer;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.plaf.FontUIResource;
import javax.swing.text.StyleContext;
import java.awt.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;

public class GUI extends JFrame {
    private JPanel mainPanel;
    private JPanel agendaPanel;
    private JPanel arrowPanel;
    private JButton previousButton;
    private JButton nextButton;
    private JButton LMButton;
    private JButton HCButton;
    private JButton UCRButton;
    private JPanel competitionButtonPanel;
    private JMenuBar menuBar = new JMenuBar();
    private JMenu menuFichier = new JMenu("File");
    private JMenuItem menuItemAddProj = new JMenuItem("Add projection");
    private JMenuItem menuItemRemoveProj = new JMenuItem("Remove projection");
    private int currentPage;
    private Agenda agenda;
    private ArrayList<Slot> slots = new ArrayList<Slot>();
    private Competition currentCompetition;
    private HashMap<String, JButton> competitionButtons = new HashMap<String, JButton>();

    public GUI() throws NotFoundInTable {
        super();
        $$$setupUI$$$();
        try {
            this.currentCompetition = new Competition("Un Certain Regard");
            construct();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    private void renderMenu() {
        menuBar.add(menuFichier);
        menuFichier.add(menuItemAddProj);
        menuFichier.add(menuItemRemoveProj);
        setJMenuBar(menuBar);
    }

    private void construct() throws NotFoundInTable {
        this.currentPage = 0;
        setTitle("Projection Planning");
        setContentPane(mainPanel);
        setSize(1280, 800);
        setLocationRelativeTo(null);
        setResizable(false);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        ImageIcon img = new ImageIcon("src/main/java/GUI/Assets/Logo/logo_cannes.jpg");
        setIconImage(img.getImage());
        this.renderMenu();
        competitionButton();
        switchAgenda(currentCompetition);
        previousButton.addActionListener(e -> nextPreviousPage(false));
        nextButton.addActionListener(e -> nextPreviousPage(true));
        menuItemAddProj.addActionListener(e -> agenda.openDialog(ProjectionType.ADD));
        menuItemRemoveProj.addActionListener(e -> agenda.openDialog(ProjectionType.REMOVE));
        setVisible(true);
    }

    private void competitionButton() {
        for (Competition competition : Competition.getAll()) {
            JButton button = new JButton(competition.getName());
            button.setUI(new PrimaryContainedButtonUI());
            button.addActionListener(actionEvent -> switchAgenda(competition));
            competitionButtonPanel.add(button);
            competitionButtons.put(competition.getName(), button);
        }
    }

    private void nextPreviousPage(boolean next) {
        if (next)
            currentPage++;
        else
            currentPage--;
        agenda = new Agenda(agendaPanel, currentPage, currentCompetition);
        currentPage = agenda.getCurrentPage();
        repaint();
        revalidate();
    }

    private void switchAgenda(Competition competition) {
        competitionButtons.get(currentCompetition.getName()).setEnabled(true);
        competitionButtons.get(competition.getName()).setEnabled(false);
        currentCompetition = competition;
        agenda = new Agenda(agendaPanel, 0, currentCompetition);
        repaint();
        revalidate();
    }

    private void createUIComponents() {
        nextButton = new JButton();
        nextButton.setUI(new PrimaryContainedButtonUI());
        previousButton = new JButton();
        previousButton.setUI(new PrimaryContainedButtonUI());
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$() {
        createUIComponents();
        mainPanel = new JPanel();
        mainPanel.setLayout(new GridLayoutManager(4, 5, new Insets(0, 0, 0, 0), -1, -1));
        agendaPanel = new JPanel();
        agendaPanel.setLayout(new BorderLayout(0, 0));
        mainPanel.add(agendaPanel, new GridConstraints(1, 0, 1, 5, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, 1, null, null, null, 0, false));
        agendaPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(0, 50, 0, 0), null, TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null));
        final Spacer spacer1 = new Spacer();
        mainPanel.add(spacer1, new GridConstraints(0, 3, 1, 2, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_WANT_GROW, 1, null, null, null, 0, false));
        arrowPanel = new JPanel();
        arrowPanel.setLayout(new GridLayoutManager(1, 2, new Insets(0, 0, 0, 0), -1, -1));
        mainPanel.add(arrowPanel, new GridConstraints(2, 4, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        Font previousButtonFont = this.$$$getFont$$$("JetBrains Mono", Font.PLAIN, -1, previousButton.getFont());
        if (previousButtonFont != null) previousButton.setFont(previousButtonFont);
        previousButton.setText("<--");
        arrowPanel.add(previousButton, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_EAST, GridConstraints.FILL_NONE, 1, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        Font nextButtonFont = this.$$$getFont$$$("JetBrains Mono", Font.PLAIN, -1, nextButton.getFont());
        if (nextButtonFont != null) nextButton.setFont(nextButtonFont);
        nextButton.setText("-->");
        arrowPanel.add(nextButton, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, 1, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final Spacer spacer2 = new Spacer();
        mainPanel.add(spacer2, new GridConstraints(3, 4, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        final JPanel panel1 = new JPanel();
        panel1.setLayout(new GridLayoutManager(1, 2, new Insets(0, 0, 0, 0), -1, -1));
        mainPanel.add(panel1, new GridConstraints(0, 0, 1, 3, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        final Spacer spacer3 = new Spacer();
        panel1.add(spacer3, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        final JPanel panel2 = new JPanel();
        panel2.setLayout(new GridLayoutManager(2, 2, new Insets(0, 0, 0, 0), -1, -1));
        panel1.add(panel2, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_NORTH, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        competitionButtonPanel = new JPanel();
        competitionButtonPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
        panel2.add(competitionButtonPanel, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_NORTH, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, new Dimension(284, 61), null, 0, false));
        final Spacer spacer4 = new Spacer();
        panel2.add(spacer4, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_WANT_GROW, 1, null, new Dimension(14, 61), null, 0, false));
        final JPanel panel3 = new JPanel();
        panel3.setLayout(new GridLayoutManager(1, 1, new Insets(0, 50, 0, 0), -1, -1));
        panel2.add(panel3, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        final JLabel label1 = new JLabel();
        Font label1Font = this.$$$getFont$$$(null, -1, 22, label1.getFont());
        if (label1Font != null) label1.setFont(label1Font);
        label1.setText("Agenda");
        panel3.add(label1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    }

    /**
     * @noinspection ALL
     */
    private Font $$$getFont$$$(String fontName, int style, int size, Font currentFont) {
        if (currentFont == null) return null;
        String resultName;
        if (fontName == null) {
            resultName = currentFont.getName();
        } else {
            Font testFont = new Font(fontName, Font.PLAIN, 10);
            if (testFont.canDisplay('a') && testFont.canDisplay('1')) {
                resultName = fontName;
            } else {
                resultName = currentFont.getName();
            }
        }
        Font font = new Font(resultName, style >= 0 ? style : currentFont.getStyle(), size >= 0 ? size : currentFont.getSize());
        boolean isMac = System.getProperty("os.name", "").toLowerCase(Locale.ENGLISH).startsWith("mac");
        Font fontWithFallback = isMac ? new Font(font.getFamily(), font.getStyle(), font.getSize()) : new StyleContext().getFont(font.getFamily(), font.getStyle(), font.getSize());
        return fontWithFallback instanceof FontUIResource ? fontWithFallback : new FontUIResource(fontWithFallback);
    }

    /**
     * @noinspection ALL
     */
    public JComponent $$$getRootComponent$$$() {
        return mainPanel;
    }
}
